# This file was generated by mettle.genes.db.GenPy3 [ver 2.1] on Mon Mar 14 20:34:08 2022
#  Target Database = postgresql
#
import asyncio
import copy
import datetime
import uuid
import time
import mettle.lib
import mettle.io
import mettle.db

from bs_monitor.db.tables.batch import tBatch
from bs_monitor.db.tables.batch_key import tBatchKey

from .batch_lock_one import dBatchLockOne
from .batch_select_one import dBatchSelectOne
from .batch_update import dBatchUpdate
from .batch_insert import dBatchInsert
from .batch_delete_one import dBatchDeleteOne

class dBatch:

    def __init__(self, dbcon: mettle.db.IConnect):
        """
        Constructor.

        :param dbcon: Mettle database connection object.
        """
        self.rec    = tBatch()
        self._dbcon = dbcon

    def __enter__(self):
        """
        DAO enter.
        """
        return self

    def __exit__(self, type, value, traceback):
        """
        DAO exit.
        """
        pass

    def lock_one(self, _lock4update: mettle.db.DBLock, _expected: bool = True) -> bool:
        """
        Lock and select the record.

        :param _lock4update: The db lock object to use.
        :param _expected: If set to false, will not raise if record is not found.
        :return: True if selected, False if not found and _expected is set to False
        """
        _key = tBatchKey()

        self.rec._write_key(_key)

        return self.lock_one_by_key(_key, _lock4update, _expected)

    def lock_one_with(self, _rec: tBatch, _lock4update: mettle.db.DBLock, _expected: bool = True) -> bool:
        """
        Lock and select the record using a record.

        :param _rec: The table rec to select with and into.
        :param _lock4update: The db lock object to use.
        :param _expected: If set to false, will not raise if record is not found.
        :return: True if selected, False if not found and _expected is set to False
        """
        _key = tBatchKey()
        _rec._write_key(_key)

        self.lock_one_by_key(_key, _lock4update, _expected)
        _rec._copy_from(self.rec)
        return True

    def lock_one_deft(self,
                      id: int,
                      _lock4update: mettle.db.DBLock,
                      _expected: bool = True) -> bool:
        """
        Attempt to lock and select the record by it's primary key columns.

        :param id: int
        :param _lock4update:  The db lock object to use. 
        :param _expected: bool
        """
        _key = tBatchKey()

        _key.id = id

        return self.lock_one_by_key(_key, _lock4update, _expected)

    def lock_one_by_key(self, _key: tBatchKey, _lock4update: mettle.db.DBLock, _expected: bool = True) -> bool:
        """
        Attempt to lock and select the record with it's primary key.

        :param _key: Primary key of the table record.
        :param _lock4update: The db lock object to use.
        :param _expected: If set to false, will not raise if record is not found.
        :return: True if selected, False if not found and _expected is set to False
        """
        with dBatchLockOne(self._dbcon) as _qry:
            _dex = _lock4update.retrys - 1 if _lock4update.retrys > 0 else 0

            while _dex >= 0:
                try:
                    _qry.exec(_key)
                    if not _qry.fetch():
                        if not _expected:
                            return False

                        raise mettle.lib.xMettle("LockOne failed to return a record (dBatch)")

                    break
                except mettle.lib.xMettle as _x:
                    if _x.get_error_code() != mettle.lib.xMettle.eCode.DBLockNoWaitFailed:
                        raise _x

                    if _dex == 0:
                        raise mettle.lib.xMettle("Could not lock Batch record after %d attempt(s) at %d mili second intervals!" % (
_lock4update.retrys, _lock4update.mili_seconds), errCode = mettle.lib.xMettle.eCode.DBLockNoWaitFailed)

                    if _lock4update.mili_seconds > 0:
                        time.sleep(_lock4update.mili_seconds * 0.001)
                _dex -= 1

            self.rec = _qry.orec

            return True

    def try_select_one(self) -> bool:
        """
        Attempt to select the record.

        :return: True if the record was found.
        """
        _key = tBatchKey()

        self.rec._write_key(_key)

        return self.try_select_one_by_key(_key)

    def try_select_one_with(self, _rec: tBatch) -> bool:
        """
        Attempt to select the record using a table rec.

        :param _rec: The table rec to select with and into.
        :return: True if the row as selectd.
        """
        _key = tBatchKey()
        _rec._write_key(_key)

        if not self.try_select_one_by_key(_key):
            return False

        _rec._copy_from(self.rec)

        return True

    def try_select_one_deft(self,
                            id: int) -> bool:
        """
        Attempt to select the record by it's primary key columns.

        :param id: int
        :return: True if the row as selectd.
        """
        _key = tBatchKey()

        _key.id = id

        return self.try_select_one_by_key(_key)

    def try_select_one_by_key(self, _key: tBatchKey) -> bool:
        """
        Attempt to select the record with it's primary key.

        :param _key: Primary key of the table record.
        :returns: True if row was selected.
        """
        with dBatchSelectOne(self._dbcon) as _qry:
            _qry.exec(_key)
            if not _qry.fetch():
                return False

            self.rec = _qry.orec

        return True

    def select_one(self) -> "dBatch":
        """
        Selects the record, raises and error if it does not exist.

        :return: Self for convenience.
        """
        if not self.try_select_one():
            raise mettle.lib.xMettle("SelectOne failed to return a record (dBatch)")

        return self

    def select_one_with(self, _rec: tBatch) -> "dBatch":
        """
        Attempt to select the record with the table record.

        :param _rec: Table record.
        :returns: Self for convenience.
        """
        if not self.try_select_one_with(_rec):
            raise mettle.lib.xMettle("SelectOne failed to return a record (dBatch)")

        return self

    def select_one_deft(self,
                        id: int) -> "dBatch":
        """
        Selects the record by its primary key columns, raises and error if it does not exist.

        :param id: int
        """
        _key = tBatchKey()

        _key.id = id

        if not self.try_select_one_by_key(_key):
            raise mettle.lib.xMettle("SelectOne failed to return a record (dBatch)")

        return self

    def select_one_by_key(self, _key: tBatchKey) -> "dBatch":
        """
        Attempt to select the record with it's primary key.

        :param _key: Primary key of the table record.
        :return: Self for convenience.
        """
        if not self.try_select_one_by_key(_key):
            raise mettle.lib.xMettle("SelectOne failed to return a record (dBatch)")

        return self

    def update_deft(self,
                    id: int,
                    parent_id: int,
                    name: str,
                    group_id: str,
                    status: str,
                    cycle: str,
                    run_date: datetime.datetime,
                    run_interval: int,
                    run_time: str,
                    ext_data: dict,
                    stamp_by: str):
        """
        Update the record by setting all column values.

        :param id: int
        :param parent_id: int
        :param name: str
        :param group_id: str
        :param status: str
        :param cycle: str
        :param run_date: datetime.datetime
        :param run_interval: int
        :param run_time: str
        :param ext_data: dict
        :param stamp_by: str
        :param stamp_tm: datetime.datetime
        """
        self.rec.id = id
        self.rec.parent_id = parent_id
        self.rec.name = name
        self.rec.group_id = group_id
        self.rec.status = status
        self.rec.cycle = cycle
        self.rec.run_date = run_date
        self.rec.run_interval = run_interval
        self.rec.run_time = run_time
        self.rec.ext_data = ext_data
        self.rec.stamp_by = stamp_by

        self.update()

    def update(self, irec: tBatch = None):
        """
        Update the record in the database with the record or irec if it is not None.

        :param irec: If not None use this record to update the database.
        """
        if irec:
            self.rec._copy_from(irec)

        with dBatchUpdate(self._dbcon) as _qry:
            _qry.exec(self.rec)

        if irec:
            irec._copy_from(self.rec)

    def insert_deft(self,
                    parent_id: int,
                    name: str,
                    group_id: str,
                    status: str,
                    cycle: str,
                    run_date: datetime.datetime,
                    run_interval: int,
                    run_time: str,
                    ext_data: dict,
                    stamp_by: str):
        """
        Insert the record setting all column values.

        :param id: int
        :param parent_id: int
        :param name: str
        :param group_id: str
        :param status: str
        :param cycle: str
        :param run_date: datetime.datetime
        :param run_interval: int
        :param run_time: str
        :param ext_data: dict
        :param stamp_by: str
        :param stamp_tm: datetime.datetime
        """
        self.rec.parent_id = parent_id
        self.rec.name = name
        self.rec.group_id = group_id
        self.rec.status = status
        self.rec.cycle = cycle
        self.rec.run_date = run_date
        self.rec.run_interval = run_interval
        self.rec.run_time = run_time
        self.rec.ext_data = ext_data
        self.rec.stamp_by = stamp_by

        self.insert()

    def insert(self, irec: tBatch = None):
        """
        Insert the record in the database with the record or irec if it is not None.

        :param irec: If not None use this record to insert into the database.
        """
        if irec:
            self.rec._copy_from(irec)

        with dBatchInsert(self._dbcon) as _qry:
            _qry.exec(self.rec)

        if irec:
            irec._copy_from(self.rec)

    def delete_one_deft(self,
                        id: int):
        """
        Delete one record by it's primary key columns.

        :param id: int
        """
        _key = tBatchKey()

        _key.id = id

        self.delete_one(_key)

    def delete_one(self, irec: tBatchKey = None):
        """
        Delete the record by its primary key or irec if it is not None.

        :param irec: If not None use this record to delete record.
        """
        _key = tBatchKey()

        if irec:
            _key._copy_from(irec)
        else:
            self.rec._write_key(_key)

        with dBatchDeleteOne(self._dbcon) as _qry:
            _qry.exec(_key)

